@@ -6,10 +6,12 @@ Testing for mean shift clustering methods
 import numpy as np
 import warnings
 
+from scipy import sparse
+
 from sklearn.utils.testing import assert_equal
 from sklearn.utils.testing import assert_false
-from sklearn.utils.testing import assert_true
 from sklearn.utils.testing import assert_array_equal
+from sklearn.utils.testing import assert_array_almost_equal
 from sklearn.utils.testing import assert_raise_message
 
 from sklearn.cluster import MeanShift
@@ -18,6 +20,7 @@ from sklearn.cluster import estimate_bandwidth
 from sklearn.cluster import get_bin_seeds
 from sklearn.datasets.samples_generator import make_blobs
 
+
 n_clusters = 3
 centers = np.array([[1, 1], [-1, -1], [1, -1]]) + 10
 X, _ = make_blobs(n_samples=300, n_features=2, centers=centers,
@@ -27,7 +30,14 @@ X, _ = make_blobs(n_samples=300, n_features=2, centers=centers,
 def test_estimate_bandwidth():
     # Test estimate_bandwidth
     bandwidth = estimate_bandwidth(X, n_samples=200)
-    assert_true(0.9 <= bandwidth <= 1.5)
+    assert 0.9 <= bandwidth <= 1.5
+
+
+def test_estimate_bandwidth_1sample():
+    # Test estimate_bandwidth when n_samples=1 and quantile<1, so that
+    # n_neighbors is set to 1.
+    bandwidth = estimate_bandwidth(X, n_samples=1, quantile=0.3)
+    assert_array_almost_equal(bandwidth, 0., decimal=5)
 
 
 def test_mean_shift():
@@ -46,6 +56,28 @@ def test_mean_shift():
     assert_equal(n_clusters_, n_clusters)
 
 
+def test_estimate_bandwidth_with_sparse_matrix():
+    # Test estimate_bandwidth with sparse matrix
+    X = sparse.lil_matrix((1000, 1000))
+    msg = "A sparse matrix was passed, but dense data is required."
+    assert_raise_message(TypeError, msg, estimate_bandwidth, X, 200)
+
+
+def test_parallel():
+    centers = np.array([[1, 1], [-1, -1], [1, -1]]) + 10
+    X, _ = make_blobs(n_samples=50, n_features=2, centers=centers,
+                      cluster_std=0.4, shuffle=True, random_state=11)
+
+    ms1 = MeanShift(n_jobs=2)
+    ms1.fit(X)
+
+    ms2 = MeanShift()
+    ms2.fit(X)
+
+    assert_array_almost_equal(ms1.cluster_centers_, ms2.cluster_centers_)
+    assert_array_equal(ms1.labels_, ms2.labels_)
+
+
 def test_meanshift_predict():
     # Test MeanShift.predict
     ms = MeanShift(bandwidth=1.2)
@@ -68,6 +100,18 @@ def test_unfitted():
     assert_false(hasattr(ms, "labels_"))
 
 
+def test_cluster_intensity_tie():
+    X = np.array([[1, 1], [2, 1], [1, 0],
+                  [4, 7], [3, 5], [3, 6]])
+    c1 = MeanShift(bandwidth=2).fit(X)
+
+    X = np.array([[4, 7], [3, 5], [3, 6],
+                  [1, 1], [2, 1], [1, 0]])
+    c2 = MeanShift(bandwidth=2).fit(X)
+    assert_array_equal(c1.labels_, [1, 1, 1, 0, 0, 0])
+    assert_array_equal(c2.labels_, [0, 0, 0, 1, 1, 1])
+
+
 def test_bin_seeds():
     # Test the bin seeding technique which can be used in the mean shift
     # algorithm
@@ -80,20 +124,20 @@ def test_bin_seeds():
     ground_truth = set([(1., 1.), (2., 1.), (0., 0.)])
     test_bins = get_bin_seeds(X, 1, 1)
     test_result = set([tuple(p) for p in test_bins])
-    assert_true(len(ground_truth.symmetric_difference(test_result)) == 0)
+    assert len(ground_truth.symmetric_difference(test_result)) == 0
 
     # With a bin coarseness of 1.0 and min_bin_freq of 2, 2 bins should be
     # found
     ground_truth = set([(1., 1.), (2., 1.)])
     test_bins = get_bin_seeds(X, 1, 2)
     test_result = set([tuple(p) for p in test_bins])
-    assert_true(len(ground_truth.symmetric_difference(test_result)) == 0)
+    assert len(ground_truth.symmetric_difference(test_result)) == 0
 
     # With a bin size of 0.01 and min_bin_freq of 1, 6 bins should be found
     # we bail and use the whole data here.
     with warnings.catch_warnings(record=True):
         test_bins = get_bin_seeds(X, 0.01, 1)
-    assert_array_equal(test_bins, X)
+    assert_array_almost_equal(test_bins, X)
 
     # tight clusters around [0, 0] and [1, 1], only get two bins
     X, _ = make_blobs(n_samples=100, n_features=2, centers=[[0, 0], [1, 1]],
