@@ -13,12 +13,10 @@ def configuration(parent_package='', top_path=None):
 
     cblas_libs, blas_info = get_blas_info()
 
-    libraries = []
     if os.name == 'posix':
         cblas_libs.append('m')
-        libraries.append('m')
 
-    config.add_extension('cd_fast', sources=['cd_fast.c'],
+    config.add_extension('cd_fast', sources=['cd_fast.pyx'],
                          libraries=cblas_libs,
                          include_dirs=[join('..', 'src', 'cblas'),
                                        numpy.get_include(),
@@ -27,10 +25,18 @@ def configuration(parent_package='', top_path=None):
                                                           []), **blas_info)
 
     config.add_extension('sgd_fast',
-                         sources=['sgd_fast.c'],
-                         include_dirs=[numpy.get_include()],
-                         libraries=libraries,
-                         )
+                         sources=['sgd_fast.pyx'],
+                         include_dirs=[join('..', 'src', 'cblas'),
+                                       numpy.get_include(),
+                                       blas_info.pop('include_dirs', [])],
+                         libraries=cblas_libs,
+                         extra_compile_args=blas_info.pop('extra_compile_args',
+                                                          []),
+                         **blas_info)
+
+    config.add_extension('sag_fast',
+                         sources=['sag_fast.pyx'],
+                         include_dirs=numpy.get_include())
 
     # add other directories
     config.add_subpackage('tests')
