@@ -1,27 +1,29 @@
 from __future__ import division
-import warnings
+
+import functools
 
 import numpy as np
 import scipy.sparse as sp
+import pytest
 
 from sklearn.metrics import euclidean_distances
 
-from sklearn.random_projection import (
-    johnson_lindenstrauss_min_dim,
-    gaussian_random_matrix,
-    sparse_random_matrix,
-    SparseRandomProjection,
-    GaussianRandomProjection)
+from sklearn.random_projection import johnson_lindenstrauss_min_dim
+from sklearn.random_projection import gaussian_random_matrix
+from sklearn.random_projection import sparse_random_matrix
+from sklearn.random_projection import SparseRandomProjection
+from sklearn.random_projection import GaussianRandomProjection
 
-from sklearn.utils.testing import (
-    assert_less,
-    assert_raises,
-    assert_raise_message,
-    assert_array_equal,
-    assert_equal,
-    assert_almost_equal,
-    assert_in,
-    assert_array_almost_equal)
+from sklearn.utils.testing import assert_less
+from sklearn.utils.testing import assert_raises
+from sklearn.utils.testing import assert_raise_message
+from sklearn.utils.testing import assert_array_equal
+from sklearn.utils.testing import assert_equal
+from sklearn.utils.testing import assert_almost_equal
+from sklearn.utils.testing import assert_in
+from sklearn.utils.testing import assert_array_almost_equal
+from sklearn.utils.testing import assert_warns
+from sklearn.exceptions import DataDimensionalityWarning
 
 all_sparse_random_matrix = [sparse_random_matrix]
 all_dense_random_matrix = [gaussian_random_matrix]
@@ -75,7 +77,7 @@ def test_input_size_jl_min_dim():
                   2 * [0.9])
 
     johnson_lindenstrauss_min_dim(np.random.randint(1, 10, size=(10, 10)),
-                                  0.5 * np.ones((10, 10)))
+                                  np.full((10, 10), 0.5))
 
 
 ###############################################################################
@@ -114,25 +116,25 @@ def check_input_with_sparse_random_matrix(random_matrix):
                       random_matrix, n_components, n_features, density=density)
 
 
-def test_basic_property_of_random_matrix():
-    """Check basic properties of random matrix generation"""
-    for random_matrix in all_random_matrix:
-        check_input_size_random_matrix(random_matrix)
-        check_size_generated(random_matrix)
-        check_zero_mean_and_unit_norm(random_matrix)
+@pytest.mark.parametrize("random_matrix", all_random_matrix)
+def test_basic_property_of_random_matrix(random_matrix):
+    # Check basic properties of random matrix generation
+    check_input_size_random_matrix(random_matrix)
+    check_size_generated(random_matrix)
+    check_zero_mean_and_unit_norm(random_matrix)
 
-    for random_matrix in all_sparse_random_matrix:
-        check_input_with_sparse_random_matrix(random_matrix)
 
-        random_matrix_dense = \
-            lambda n_components, n_features, random_state: random_matrix(
-                n_components, n_features, random_state=random_state,
-                density=1.0)
-        check_zero_mean_and_unit_norm(random_matrix_dense)
+@pytest.mark.parametrize("random_matrix", all_sparse_random_matrix)
+def test_basic_property_of_sparse_random_matrix(random_matrix):
+    check_input_with_sparse_random_matrix(random_matrix)
+
+    random_matrix_dense = functools.partial(random_matrix, density=1.0)
+
+    check_zero_mean_and_unit_norm(random_matrix_dense)
 
 
 def test_gaussian_random_matrix():
-    """Check some statical properties of Gaussian random matrix"""
+    # Check some statical properties of Gaussian random matrix
     # Check that the random matrix follow the proper distribution.
     # Let's say that each element of a_{ij} of A is taken from
     #   a_ij ~ N(0.0, 1 / n_components).
@@ -146,7 +148,7 @@ def test_gaussian_random_matrix():
 
 
 def test_sparse_random_matrix():
-    """Check some statical properties of sparse random matrix"""
+    # Check some statical properties of sparse random matrix
     n_components = 100
     n_features = 500
 
@@ -212,7 +214,7 @@ def test_sparse_random_projection_transformer_invalid_density():
 def test_random_projection_transformer_invalid_input():
     for RandomProjection in all_RandomProjection:
         assert_raises(ValueError,
-                      RandomProjection(n_components='auto').fit, [0, 1, 2])
+                      RandomProjection(n_components='auto').fit, [[0, 1, 2]])
 
         assert_raises(ValueError,
                       RandomProjection(n_components=-10).fit, data)
@@ -338,8 +340,18 @@ def test_warning_n_components_greater_than_n_features():
     data, _ = make_sparse_random_data(5, n_features, int(n_features / 4))
 
     for RandomProjection in all_RandomProjection:
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-            RandomProjection(n_components=n_features + 1).fit(data)
-            assert_equal(len(w), 1)
-            assert issubclass(w[-1].category, UserWarning)
+        assert_warns(DataDimensionalityWarning,
+                     RandomProjection(n_components=n_features + 1).fit, data)
+
+
+def test_works_with_sparse_data():
+    n_features = 20
+    data, _ = make_sparse_random_data(5, n_features, int(n_features / 4))
+
+    for RandomProjection in all_RandomProjection:
+        rp_dense = RandomProjection(n_components=3,
+                                    random_state=1).fit(data)
+        rp_sparse = RandomProjection(n_components=3,
+                                     random_state=1).fit(sp.csr_matrix(data))
+        assert_array_almost_equal(densify(rp_dense.components_),
+                                  densify(rp_sparse.components_))
